# 入口类型
EXECUTOR_TYPE_START = 'Start'
# 结束类型
EXECUTOR_TYPE_END = 'End'
# 判断类型
EXECUTOR_TYPE_JUDGE = 'Judge'
# 常规类型
EXECUTOR_TYPE_NORMAL = 'Normal'
# 未知类型
EXECUTOR_TYPE_UNKNOW = 'Unknow'

# 默认的输入 handle
DEFAULT_INPUT_HANDLE_VALUE = 'input'
DEFAULT_INPUT_HANDLE = {'handle': DEFAULT_INPUT_HANDLE_VALUE, 'title': 'input'}
# 默认的输出 handle
DEFAULT_OUTPUT_HANDLE_VALUE = 'output'
DEFAULT_OUTPUT_HANDLE = {'handle': DEFAULT_OUTPUT_HANDLE_VALUE, 'title': 'output'}
